#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# postes_interdits.py
#
# Génération des règles iptables
# pour l'interdiction réseau ou web
# des postes et des sous-réseaux
#
###########################################################################

import os
from creole.parsedico import parse_dico

import pickle

# fichiers à utiliser
fpostes = "/var/www/ead/tmp/poste.txt"
fdestination = "/var/www/ead/tmp/destination.txt"
fhoraires = "/var/www/ead/tmp/horaires_ip.txt"

# Traduction des jours
jour = dict(lundi='Mon', mardi='Tue', mercredi='Wed', jeudi='Thu',
            vendredi='Fri', samedi='Sat', dimanche='Sun')

# Liste des jours
semaine = ['Mon','Tue','Wed','Thu','Fri','Sat','Sun']

# règles IPTABLES pour les postes à écarter du réseau
def regles_poste(poste) :
    return """/sbin/iptables -t nat -I PREROUTING -s %s -j DROP
/sbin/iptables -I INPUT -s %s -j DROP""" % (poste, poste)

# règles IPTABLES pour les destinations à écarter du réseau
def regles_destination(poste) :
    return """/sbin/iptables  -I FORWARD -d %s -j DROP
/sbin/iptables -I INPUT -s %s -j DROP""" % (poste, poste)

# règles IPTABLES pour les accès Internet
def regles_horaires(ligne, gw) :
    try:
        dico = dict(gw = gw)
        dico['poste'], dico['tstart'], dico['tstop'], dico['jdeb'], dico['jfin'] =ligne.split("#", 5)
    except ValueError:
        return None
    regle = """/sbin/iptables -I INPUT -p tcp --dport 3128 -s %(poste)s -m time --timestart %(tstart)s --timestop %(tstop)s --days %%s -j DROP
/sbin/iptables -I FORWARD -o %(gw)s -s %(poste)s -m time --timestart %(tstart)s --timestop %(tstop)s --days %%s -j DROP
"""%dico
    jliste = get_jliste(dico['jdeb'], dico['jfin'])
    return regle%(jliste, jliste)

def get_jliste(jdeb, jfin):
    istart, iend = semaine.index(jour[jdeb]), semaine.index(jour[jfin])
    if istart < iend:
        jliste = ','.join(semaine[istart: iend+1])
    elif istart == iend:
        jliste = semaine[istart]
    else:
        jliste = ','.join(semaine[istart:])
        jliste += ','
        jliste += ','.join(semaine[:iend+1])
    return jliste


# programme principal
def main():
    # traitement du fichier des postes interdits
    try:
        if os.path.isfile(fpostes):
            for ligne in file(fpostes,'r').readlines():
                r = regles_poste(ligne.strip())
                os.system(r)
    except Exception, e:
        try: print 'Erreur : %s'%e
        except: pass
    # traitement du fichier destinations interdites    
    try:
        if os.path.isfile(fdestination):
            for ligne in file(fdestination,'r').readlines():
                r = regles_destination(ligne.strip())
                os.system(r)
    except Exception, e:
        try: print 'Erreur : %s'%e
        except: pass
    # traitement du fichier des postes soumis à des horaires
    try:
        if os.path.isfile(fhoraires):
            gw = parse_dico()["interface_gw"]
            for ligne in file(fhoraires).readlines():
                r = regles_horaires(ligne.strip(), gw)
                if r:
                    os.system(r)
    except:
        pass
    # gestion des groupes de postes
    set_group_rules()

def set_group_rules():
    # traitement du fichier des groupes de postes
    try:
        from ead2.backend.actions.lib.ipset import iptables_generator
        a = iptables_generator.IptablesGenerator()
        a.create_rules()
    except:
        pass

if __name__ == '__main__':
    main()
